<?php
/*
Plugin Name: Pay3 Payment Gateway
Description: Integrates Pay3 payment gateway for shop or financial websites.
Version: 1.0
Author: Pay3.ir
*/

class Pay3PaymentGateway {

    private $gatewayUrl = 'https://pay3.ir/api/payment/make';

    public function __construct() {
        add_action('admin_menu', [$this, 'addAdminMenu']);
        add_action('admin_init', [$this, 'registerSettings']);
        add_shortcode('pay3_payment_button', [$this, 'renderPaymentButton']);
    }

    /**
     * Add Pay3 settings page to the WordPress admin menu
     */
    public function addAdminMenu() {
        add_options_page('Pay3 Settings', 'Pay3 Payment', 'manage_options', 'pay3-settings', [$this, 'settingsPage']);
    }

    /**
     * Register plugin settings
     */
    public function registerSettings() {
        register_setting('pay3_settings_group', 'pay3_gateway_hash');
    }

    /**
     * Render settings page
     */
    public function settingsPage() {
        ?>
        <div class="wrap">
            <h1>Pay3 Payment Gateway Settings</h1>
            <form method="post" action="options.php">
                <?php settings_fields('pay3_settings_group'); ?>
                <?php do_settings_sections('pay3_settings_group'); ?>
                <table class="form-table">
                    <tr valign="top">
                        <th scope="row">Gateway Hash</th>
                        <td><input type="text" name="pay3_gateway_hash" value="<?php echo esc_attr(get_option('pay3_gateway_hash')); ?>" size="50" /></td>
                    </tr>
                </table>
                <?php submit_button(); ?>
            </form>
        </div>
        <?php
    }

    /**
     * Render payment button shortcode
     */
    public function renderPaymentButton($atts) {
        $attributes = shortcode_atts(['amount' => ''], $atts);

        if (empty($attributes['amount']) || !is_numeric($attributes['amount'])) {
            return '<p>Error: Invalid or missing amount.</p>';
        }

        $amount = intval($attributes['amount']);
        $callbackUrl = home_url('/pay3-callback');
        
        return $this->initiateTransaction($amount, $callbackUrl);
    }

    /**
     * Initiate a transaction with Pay3
     */
    private function initiateTransaction($amount, $callbackUrl) {
        $gatewayHash = get_option('pay3_gateway_hash');

        if (!$gatewayHash) {
            return '<p>Error: Gateway hash is not configured.</p>';
        }

        try {
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $this->gatewayUrl);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query([
                'gateway' => $gatewayHash,
                'amount' => $amount,
                'callback_url' => $callbackUrl,
            ]));
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type: application/x-www-form-urlencoded']);

            $response = curl_exec($ch);

            if (curl_errno($ch)) {
                throw new Exception('cURL Error: ' . curl_error($ch));
            }

            curl_close($ch);

            $responseData = json_decode($response, true);

            if (!empty($responseData['has_error']) && $responseData['has_error']) {
                return '<p><font color="red">Error: ' . $this->getErrorMessage($responseData['error_code']) . '</font></p>';
            }

            if (isset($responseData['url'])) {
                return '<a class="pay3_btn btn button" href="' . esc_url($responseData['url']) . '">پرداخت</a>';
            }

            throw new Exception("Unknown error: Unable to process payment.");
        } catch (Exception $e) {
            return '<p><font color="red">Error: ' . $e->getMessage() . '</font></p>';
        }
    }

    /**
     * Get human-readable error messages based on error codes
     */
    private function getErrorMessage($errorCode) {
        switch ($errorCode) {
            case 2:
                return 'Required parameters are missing.';
            case 3:
                return 'Parameters contain invalid values.';
            case 4:
                return 'Gateway is empty.';
            case 5:
                return 'Amount is not specified.';
            case 6:
                return 'Callback URL is not provided.';
            case 7:
                return 'Amount is not valid.';
            case 8:
                return 'Callback URL is not valid.';
            case 9:
                return 'Gateway not found.';
            case 10:
                return 'Account is not accepted.';
            case 11:
                return 'Gateway is not accepted.';
            case 12:
                return 'Insufficient wallet balance.';
            default:
                return 'An unknown error occurred. Code: ' . $errorCode;
        }
    }
}

new Pay3PaymentGateway();
